import java.util.Enumeration;
import jds.collection.LinkedList;
import jds.util.Comparable;

public class LargePositiveInteger implements Comparable {
	LargePositiveInteger (int start) {
		while (start > 0) {
			digits.addLast(new Integer(start%1000));
			start = start / 1000;
		}
	}

	private LargePositiveInteger (LinkedList d) { digits = d; }

	private LinkedList digits = new LinkedList();

	public String toString() {
		if (digits.isEmpty()) return "0";
		String result = "";
		Enumeration e = digits.elements();
		while (e.hasMoreElements()) {
			String block = "000" + e.nextElement();
			result = block.substring(block.length()-3) + result;
			}
		return result;
	}

	public LargePositiveInteger add (LargePositiveInteger right) {
		makeSameSize(right);
		Enumeration le = digits.elements();
		Enumeration re = right.digits.elements();
		LinkedList result = new LinkedList();
		int sum = 0;
		int carry = 0;
		while (le.hasMoreElements() && le.hasMoreElements()) {
			Integer leftDigit = (Integer) le.nextElement();
			Integer rightDigit = (Integer) re.nextElement();
			sum = leftDigit.intValue() + rightDigit.intValue() + carry;
			carry = sum / 1000;
			result.addLast(new Integer(sum % 1000));
		}
		if (carry != 0)
			result.addLast(new Integer(carry));
		return new LargePositiveInteger(result);
	}

	private void makeSameSize (LargePositiveInteger right) {
		while (digits.size() < right.digits.size())
			digits.addLast(new Integer(0));
		while (right.digits.size() < digits.size())
			right.digits.addLast(new Integer(0));
	}
	public int compareTo (Object r) { 
		LargePositiveInteger right = (LargePositiveInteger) r;
		makeSameSize(right);
		int result = 0;
		Enumeration le = digits.elements();
		Enumeration re = right.digits.elements();
		while (le.hasMoreElements() && le.hasMoreElements()) {
			Integer leftDigit = (Integer) le.nextElement();
			Integer rightDigit = (Integer) re.nextElement();
			int ld = leftDigit.intValue();
			int rd = rightDigit.intValue();
			if (ld < rd) result = -1;
			if (ld > rd) result = 1;
		}
		return result;
	}

	public LargePositiveInteger sub (LargePositiveInteger right) {
		makeSameSize(right);
		Enumeration le = digits.elements();
		Enumeration re = right.digits.elements();
		LinkedList result = new LinkedList();
		int borrow = 0;
		int carry = 0;
		while (le.hasMoreElements() && le.hasMoreElements()) {
			Integer leftDigit = (Integer) le.nextElement();
			Integer rightDigit = (Integer) re.nextElement();
			int ld = leftDigit.intValue();
			int rd = rightDigit.intValue();
			ld = ld - borrow;
			borrow = 0;
			if (ld < rd) {
				ld = ld + 1000;
				borrow = 1;
				}
			result.addLast(new Integer(ld - rd));
		}
		return new LargePositiveInteger(result);
	}

	public static void main (String [ ] args) {
		LargePositiveInteger a = new LargePositiveInteger(11998);
		System.out.println("a is " + a);
		LargePositiveInteger b = new LargePositiveInteger(9999);
		System.out.println("b is " + b);
		LargePositiveInteger c = a.sub(b);
		System.out.println("c is " + c);
		System.out.println("compare " + a.compareTo(b));
	}
}

