import java.awt.*;
import java.awt.event.*;
import jds.Queue;
import jds.collection.LinkedList;
import jds.util.CloseQuit;

class Rain extends Frame {
	public static void main (String [ ] args) {
		Rain world = new Rain();
		world.show();
		world.run();
	}

	public Rain () {
			// window initialization
		setSize(FrameWidth, FrameHeight);
		setTitle("Rain Game");
		addWindowListener(new CloseQuit());

			// application specific initialization
		for (int i = 0; i < BoardHeight; i++)
			for (int j = 0; j < BoardWidth; j++)
				grid[i][j] = Color.white;
		addKeyListener (new keyDown());
	}

	private Color [ ] [ ] grid = new Color[BoardHeight][BoardWidth];
	private Queue commandQueue = new LinkedList();
	private Piece piece = new Piece(3, 3, BoardHeight-1);
	final static int FrameWidth = 150;
	final static int FrameHeight = 330;
	final static int BoardHeight = 15;
	final static int BoardWidth = 5;
	final static int PieceWidth = 20;

	public void paint (Graphics g) {
		for (int i = 0; i < BoardHeight; i++)
			for (int j = 0; j < BoardWidth; j++) {
				g.setColor(grid[i][j]);
				g.fillRect(PieceWidth*j, 
					FrameHeight-PieceWidth*(i+1), 
					PieceWidth, PieceWidth);
				}
	}

	public void run () {
		boolean loop = true;
		while (loop) {
			if (piece.move() == false) {
				int ic = (int) (4 * Math.random());
				int iy = BoardHeight-1;
				int ix = (int) (BoardWidth * Math.random());
				if (grid[iy][ix].equals(Color.white)) {
					piece = new Piece(ic, ix, iy);
					}
				else loop = false;
				}
			repaint();
			try { Thread.sleep(200); } catch(Exception e) { }
		}
			// game over
		System.out.println("game over");
	}

	private class keyDown extends KeyAdapter {
		public void keyPressed (KeyEvent e) {
			char key = e.getKeyChar();
			switch (key) {
				case 'a': commandQueue.addLast(new Character('l')); break;
				case 'd': commandQueue.addLast(new Character('r')); break;
				case 'q': System.exit(0);
			}
		}
	}

	private class Piece /* extends Thread */ {
		Piece (int v, int ix, int iy) {
			switch (v) {
				case 0: color = Color.red; break;
				case 1: color = Color.blue; break;
				case 2: color = Color.green; break;
				default: color = Color.yellow; break;
			}
			y = iy;
			x = ix;
			grid[y][x] = color;
		}

		private Color color;
		private int x, y;

		public boolean move () {
			if (! commandQueue.isEmpty()) {
				Character command = 
					(Character) commandQueue.getFirst();
				commandQueue.removeFirst();
				if (command.charValue() == 'r')
					moveRight();
				else moveLeft();
			} else {
				if ((y > 0) && grid[y-1][x].equals(Color.white)) {
					grid[y][x] = Color.white;
					y = y - 1;
					grid[y][x] = color;
				} // cannot move down, halt
				else return false;
			}
			return true;
		}

		private void moveRight() {
			if ((x < (BoardWidth-1)) && grid[y][x+1].equals(Color.white)) {
				grid[y][x] = Color.white;
				x = x + 1;
				grid[y][x] = color;
			}
		}

		private void moveLeft() {
			if ((x > 0) && grid[y][x-1].equals(Color.white)) {
				grid[y][x] = Color.white;
				x = x - 1;
				grid[y][x] = color;
			}
		}
	}
}

